__all__ = [
    '__title__', '__summary__', '__uri__', '__version__', '__author__',
    '__email__', '__license__', '__copyright__',
]

__title__ = 'django-celery-email-reboot'
__summary__ = 'An async Django email backend using celery (forked from django-celery-email)'
__uri__ = 'https://github.com/Panevo/django-celery-email'

__version__ = '4.2.0'

__originalauthor__ = 'Paul McLanahan'
__author__ = 'Karlo Krakan'
__email__ = 'karlo.krakan@panevo.com'

__license__ = 'BSD'
__copyright__ = 'Copyright 2015 {0}'.format(__originalauthor__)
